@@ -297,110 +297,6 @@ main = BenchmarkProgram
 
 
 
-# NOTE: Needed since standard parsers provided by `ConfigParser' module do not preserve
-#       order.
-class _ConfigurationParser (object):
-
-    def __init__(self):
-        self.__values = []
-
-
-    def read (self, filename):
-        if filename is None:
-            return
-
-        file = open (filename)
-
-        try:
-            section_name = None
-
-            for line in file:
-                line = line.strip ()
-
-                if not line or line[0] == '#':
-                    continue
-
-                if line[0] == '[' and line[-1] == ']':
-                    section_name = line[1:-1]
-
-                    try:
-                        self.add_section (section_name)
-                    except:
-                        pass
-
-                    continue
-
-                if section_name is not None:
-                    name_value = line.split ('=', 2)
-
-                    if len (name_value) == 2:
-                        try:
-                            self.set (section_name, name_value[0].strip (), value.strip ())
-                        except:
-                            pass
-
-        finally:
-            file.close ()
-            print self.__values
-
-
-    def write (self, filename):
-        file = open (filename, 'w')
-
-        try:
-            for section_name, values in self.__values:
-                file.write ('[%s]\n' % section_name)
-
-                for name, value in values:
-                    file.write ('%s = %s\n' % (name, value))
-
-                file.write ('\n')
-
-        finally:
-            file.close ()
-
-
-    def sections (self):
-        return [section_name for section_name, values in self.__values]
-
-
-    def has_section (self, section_name):
-        return section_name in self.sections ()
-
-
-    def add_section (self, section_name):
-        if not self.has_section (section_name):
-            self.__values.append ((section_name, []))
-        else:
-            raise ValueError ("there is already section named `%s'" % section_name)
-
-
-    def items (self, section_name):
-        for _name, values in self.__values:
-            if _name == section_name:
-                return tuple (values)
-
-        raise ValueError ("there is no section named `%s'" % section_name)
-
-
-    def set (self, section_name, value_name, new_value):
-        print self.__values
-        for _name, values in self.__values:
-            if _name == section_name:
-                for index, name_value in enumerate (values):
-                    if name_value[0] == value_name:
-                        values[index] = (value_name, new_value)
-                        break
-                else:
-                    values.append ((value_name, new_value))
-
-                print values, self.__values
-                return
-
-        raise ValueError ("there is no section named `%s'" % section_name)
-
-
-
 # Local variables:
 # mode: python
 # python-indent: 4
