@@ -790,26 +790,73 @@ class AbstractSignal (object):
     """
 
 
-    def __repr__(self):
-        return self.__to_string (self.__class__.__name__)
+    def _additional_description (self, formatter):
+        """
+        Generate list of additional descriptions for this object.  All description strings
+        are put in parentheses after basic signal description and are separated by
+        semicolons.  Default description mentions number of handlers if there are any at
+        all.
 
-    def __str__(self):
-        return self.__to_string (self.__class__.__name__.split ('.') [-1])
+        C{formatter} is either C{repr} or C{str} and should be used to format objects
+        mentioned in list string(s).  Its use is not required but encouraged.
 
+        Overriden method should look like this:
+
+            >>> def _additional_description (self, formatter):
+            ...     return (['my-description']
+            ...             + super (..., self)._additional_description (formatter))
+
+        You may selectively remove descriptions generated by superclasses, but remember
+        that some of them (including this class) may generate varying number of
+        descriptions, so this may be not trivial to do.  In general, there are no
+        requirements on contents of returned list, except that it must contain only
+        strings.
+
+        This method is called by standard implementations of C{L{__repr__}} and
+        C{L{__str__}}.  If you use your own (and that is perfectly fine), you don’t need
+        to override this method.
+
+        @param formatter: function (either C{repr} or C{str}) that can be used to format
+                          various objects.
+
+        @rtype:           C{list}
+        @returns:         List of description strings for this object.
+        """
 
-    def __to_string (self, class_name):
         try:
             num_handlers = self.count_handlers ()
 
-            if num_handlers > 0:
-                handler_data = ' (%d handlers)' % num_handlers
-            else:
-                handler_data = ''
+            if num_handlers > 1:
+                return ['%d handlers' % num_handlers]
+            elif num_handlers == 1:
+                return ['1 handler']
 
         except NotImplementedError:
-            handler_data = ' (count_handlers() not implemented)'
+            return 'count_handlers() not implemented'
 
-        return '<%s object at 0x%x%s>' % (class_name, id (self), handler_data)
+        return []
+
+    def __to_string (self, strict):
+        if strict:
+            additional_description = self._additional_description (repr)
+        else:
+            additional_description = self._additional_description (str)
+
+        if additional_description:
+            return ' (%s)' % '; '.join (additional_description)
+        else:
+            return ''
+
+
+    def __repr__(self):
+        # It is impossible to recreate signal, so don't try to generate a valid Python
+        # expression.
+        return ('<%s.%s at 0x%x%s>'
+                % (self.__module__, self.__class__.__name__, id (self), self.__to_string (True)))
+
+
+    def __str__(self):
+        return '<%s at 0x%x%s>' % (self.__class__.__name__, id (self), self.__to_string (True))
 
 
 
@@ -851,7 +898,7 @@ class Signal (AbstractSignal):
         super (Signal, self).__init__()
 
         self._handlers         = None
-        self._blocked_handlers = ()
+        self._blocked_handlers = _EMPTY_TUPLE
         self.__accumulator     = accumulator
         self.__emission_level  = 0
 
@@ -899,7 +946,7 @@ class Signal (AbstractSignal):
 
 
     def is_blocked (self, handler, *arguments):
-        if self._blocked_handlers is not () and callable (handler):
+        if self._blocked_handlers is not _EMPTY_TUPLE and callable (handler):
             if arguments:
                 handler = Binding (handler, arguments)
 
@@ -935,7 +982,7 @@ class Signal (AbstractSignal):
                 else:
                     self._handlers[index] = None
 
-                if (self._blocked_handlers is not ()
+                if (    self._blocked_handlers is not _EMPTY_TUPLE
                     and handler not in self._handlers[index:]):
                     # This is the last handler, need to make sure it is not listed in
                     # `_blocked_handlers'.
@@ -943,7 +990,7 @@ class Signal (AbstractSignal):
                                               if _handler != handler]
 
                     if not self._blocked_handlers:
-                        self._blocked_handlers = ()
+                        self._blocked_handlers = _EMPTY_TUPLE
 
                 if not self._handlers:
                     self._handlers = None
@@ -978,12 +1025,12 @@ class Signal (AbstractSignal):
                     self._handlers[index] = None
                     any_removed           = True
 
-        if any_removed and self._blocked_handlers is not ():
+        if any_removed and self._blocked_handlers is not _EMPTY_TUPLE:
             self._blocked_handlers = [_handler for _handler in self._blocked_handlers
                                       if _handler != handler]
 
             if not self._blocked_handlers:
-                self._blocked_handlers = ()
+                self._blocked_handlers = _EMPTY_TUPLE
 
         return any_removed
 
@@ -997,7 +1044,7 @@ class Signal (AbstractSignal):
                 handler = Binding (handler, arguments)
 
             if handler in self._handlers:
-                if self._blocked_handlers is not ():
+                if self._blocked_handlers is not _EMPTY_TUPLE:
                     self._blocked_handlers.append (handler)
                 else:
                     self._blocked_handlers = [handler]
@@ -1008,7 +1055,7 @@ class Signal (AbstractSignal):
 
 
     def unblock (self, handler, *arguments):
-        if self._blocked_handlers is () or not callable (handler):
+        if self._blocked_handlers is _EMPTY_TUPLE or not callable (handler):
             return False
 
         if arguments:
@@ -1018,7 +1065,7 @@ class Signal (AbstractSignal):
             self._blocked_handlers.remove (handler)
 
             if not self._blocked_handlers:
-                self._blocked_handlers = ()
+                self._blocked_handlers = _EMPTY_TUPLE
 
             return True
 
@@ -1053,7 +1100,6 @@ class Signal (AbstractSignal):
                         continue
 
                     if self.__emission_level < 0:
-                        self.__emission_level = -self.__emission_level
                         break
 
                     try:
@@ -1096,6 +1142,9 @@ class Signal (AbstractSignal):
 
 
     def collect_garbage (self):
+        # NOTE: If, for some reason, you change this, don't forget to adjust
+        #       `CleanSignal.collect_garbage' accordingly.
+
         # Don't remove disconnected or garbage-collected handlers if in nested emission,
         # it will spoil emit() calls completely.
         if self._handlers is not None and self.__emission_level == 0:
@@ -1107,6 +1156,15 @@ class Signal (AbstractSignal):
                 self._handlers = None
 
 
+    def _additional_description (self, formatter):
+        if self.__accumulator is not None:
+            descriptions = ['accumulator: %s' % formatter (self.__accumulator)]
+        else:
+            descriptions = []
+
+        return descriptions + super (Signal, self)._additional_description (formatter)
+
+
 
 class CleanSignal (Signal):
 
@@ -1196,11 +1254,34 @@ class CleanSignal (Signal):
 
     def collect_garbage (self):
         if self._handlers is not None and self._get_emission_level () == 0:
-            super (CleanSignal, self).collect_garbage ()
-            if self._handlers is None and self.__parent is not None:
+            # NOTE: This is essentially inlined method of the superclass.  While calling
+            #       that method would be more proper, inlining it gives significant speed
+            #       improvement.  Since it makes no difference for derivatives, we
+            #       sacrifice "do what is right" principle in this case.
+
+            self._handlers = [handler for handler in self._handlers
+                              if handler is not None and (not isinstance (handler, WeakBinding)
+                                                          or handler)]
+
+            if not self._handlers:
+                self._handlers = None
                 AbstractGCProtector.default.unprotect (self)
 
 
+    def _additional_description (self, formatter):
+        if self.__parent is not None:
+            descriptions = ['parent: %s' % formatter (self.__parent ())]
+        else:
+            descriptions = []
+
+        return descriptions + super (CleanSignal, self)._additional_description (formatter)
+
+
+
+# It is not guaranteed to be a singleton, although it probably always is.
+_EMPTY_TUPLE = ()
+
+
 
 # Local variables:
 # mode: python
