@@ -151,7 +151,7 @@ class LogicConditionTestCase (NotifyTestCase):
         self.assert_results (False, True, False, True, False)
 
 
-    def test_if_else (self):
+    def test_if_else_1 (self):
         self.results = []
 
         condition1 = Condition (False)
@@ -200,6 +200,22 @@ class LogicConditionTestCase (NotifyTestCase):
         self.assert_results (False, True, False, True, False, True)
 
 
+    # Test for a real bug introduced when optimizing if-else condition.
+    def test_if_else_2 (self):
+        self.results = []
+
+        condition1 = Condition (False)
+        condition2 = Condition (False)
+        condition3 = Condition (True)
+
+        if_else_condition = condition1.if_else (condition2, condition3)
+        if_else_condition.store (self.simple_handler)
+
+        condition1.state = True
+
+        self.assert_results (True, False)
+
+
 
 class PredicateConditionTestCase (NotifyTestCase):
 
@@ -555,6 +571,58 @@ class SignalConditionTestCase (NotifyTestCase):
 
 
 
+class ConditionDerivationTestCase (NotifyTestCase):
+
+    def test_derivation_1 (self):
+        DerivedCondition = \
+            AbstractStateTrackingCondition.derive_type ('DerivedCondition',
+                                                        setter = lambda condition, state: None)
+
+        condition = DerivedCondition (False)
+        self.assert_(not condition.state)
+
+        condition.set (True)
+        self.assert_(condition.state)
+
+        condition.state = False
+        self.assert_(not condition.state)
+
+
+    def test_derivation_2 (self):
+        self.results = []
+
+        DerivedCondition = \
+            AbstractStateTrackingCondition.derive_type ('DerivedCondition',
+                                                        getter = lambda condition: False,
+                                                        setter = (lambda condition, state:
+                                                                      self.simple_handler (state)))
+
+        condition = DerivedCondition ()
+        condition.set (True)
+        condition.state = False
+
+        # The default state is retrieved with the getter function, so the setter must not
+        # be called during condition creation.
+        self.assert_results (True, False)
+
+
+    def test_derivation_3 (self):
+        self.results = []
+
+        DerivedCondition = \
+            AbstractStateTrackingCondition.derive_type ('DerivedCondition',
+                                                        setter = (lambda condition, state:
+                                                                      self.simple_handler (state)))
+
+        condition = DerivedCondition (False)
+        condition.set (True)
+        condition.state = False
+
+        # There is no getter at all, so setter must be called during condition creation.
+        self.assert_results (False, True, False)
+
+
+
 if __name__ == '__main__':
     unittest.main ()
 
